unit Datum;

interface

uses
  Helyes, Rtti;

const
	Januar : integer = 1;
	Februar : integer = 2;
	Marcius : integer = 3;
	Aprilis : integer = 4;
	Majus : integer = 5;
	Junius : integer = 6;
	Julius : integer = 7;
	Augusztus : integer = 8;
	Szeptember : integer = 9;
	Oktober : integer = 10;
	November : integer = 11;
	December : integer = 12;

type
  { A TDatum osztly annotlsa TInvarins attribtummal, a paramter az invarins ellenrz fggvny neve }
  [TInvarians('DatumInvarians')]
  TDatum = class
    strict protected
      FHo: integer;
      FNap: integer;
      FEv: integer;
      HelyessegBizonyito: THelyessegBizonyito<TDatum>;
      procedure SetEv(const Value: integer); virtual;
      procedure SetHo(const Value: integer); virtual;
      procedure SetNap(const Value: integer); virtual;
      function GetSzokoEv : boolean; virtual;
    public
      constructor Create (iEv, iHo, iNap : integer); overload;
      constructor Create (iDatum : TDatum); overload;
      constructor CreateMasnap (iDatum : TDatum);
      property Ev : integer read FEv write SetEv;
      property Ho : integer read FHo write SetHo;
      property Nap : integer read FNap write SetNap;
      property SzokoEv : boolean read GetSzokoEv;
      procedure	MasnapraLep; virtual;
      function Masnap : TDatum; virtual;
      { A NapokSzamaAHonapban fggvny annotlsa TUtoFelt attribtummal, a paramter az utfelttel ellenrz fggvny neve }
      [TUtoFelt('NapokSzamaUtoFelt')]
      function NapokSzamaAHonapban : integer; virtual;
      function DatumInvarians : boolean;
      function NapokSzamaUtoFelt : boolean;
  end;

implementation

{ Tdatum }

{ Az invarins ellenrz fggvny }
function TDatum.DatumInvarians : boolean;
begin
  Result := (FHo >= Januar) and
  (FHo <= December) and
	(FNap >= 1) and
	(FNap <= 31);
  if not Result then
    WriteLn('Az invarians serult!!!');
end;

constructor TDatum.Create (iEv, iHo, iNap : integer);
begin
  Ev := iEv;
  Ho := iHo;
  Nap := iNap;
  HelyessegBizonyito := THelyessegBizonyito<TDatum>.Create (Self);
end;

constructor TDatum.Create (iDatum : TDatum);
begin
  Create (iDatum.Ev, iDatum.Ho, iDatum.Nap);
end;

constructor TDatum.CreateMasnap (iDatum : TDatum);
begin
  Create (iDatum);
  MasnapraLep;
end;

procedure TDatum.MasnapraLep;
begin
  if FNap = NapokSzamaAHonapban then begin
    if Ho = December then begin
      Ev := Ev + 1;
      Ho := Januar;
      Nap := 1;
    end
    else begin
      Ho := Ho + 1;
      Nap := 1;
    end
  end
  else
    Nap := Nap + 1;
end;

function TDatum.Masnap : TDatum;
begin
  Result := TDatum.CreateMasnap (Self);
end;

{ A NapokSzamaAHonapban metdus utfelttelt ellenrz fggvny }
function TDatum.NapokSzamaUtoFelt : boolean;
begin
  Result := (FNap >= 1) and
  (FNap <= 31);
  if not Result then
    WriteLn('Az utofeltetel serult!!!');
end;

function TDatum.NapokSzamaAHonapban : integer;
begin
	case Ho of
		2 : if SzokoEv then Result := 29
				else Result := 28;
		4,6,9,11 : Result := 30;
    else Result := 31;
  end;
end;

function TDatum.GetSzokoEv : boolean;
begin
		if ev mod 400 = 0 then Result := true
		else
      if ev mod 100 = 0 then Result := false
		  else
        if ev mod 4 = 0 then Result := true
		    else Result := false;
end;

procedure TDatum.SetEv(const Value: integer);
begin
  FEv := Value;
end;

procedure TDatum.SetHo(const Value: integer);
begin
  FHo := Value;
end;

procedure TDatum.SetNap(const Value: integer);
begin
  FNap := Value;
end;

end.
