unit Helyes;
{$M+}
interface

uses
  Rtti;

type
  { Az annotcihoz hasznlt objektum }
  THBAttr = class(TCustomAttribute)
    public
      Vizsgal : string;
    published
      constructor Create (Feltetel : string);
  end;

  TInvarians = class(THBAttr)
  end;

  TEloFelt = class(THBAttr)
  end;

  TUtoFelt = class(THBAttr)
  end;

  { A helyessgbizonyt osztly deklarcija }
  THelyessegBizonyito<T : class> = class
	  strict protected
	    FInterceptor: TVirtualMethodInterceptor;
	    FInstance: T;
	  public
	    constructor Create(Instance: T);
            procedure Felold (Instance : T);
	  end;


implementation

{ THBAttr }

constructor THBAttr.Create(Feltetel: string);
begin
  Vizsgal := Feltetel;
end;

{ THelyessegBizonyito<T> }

constructor THelyessegBizonyito<T>.Create(Instance: T);
begin
  inherited Create;
  FInstance := Instance;
  FInterceptor := TVirtualMethodInterceptor.Create(Instance.ClassType);
  { Az OnBefore esemny belltsa az itt deklarlt nvtelen eljrsra }
  FInterceptor.OnBefore :=
      procedure(Instance: TObject; Method: TRttiMethod;
          const Args: TArray<TValue>; out DoInvoke: Boolean; out Result: TValue)
      var
        LContext: TRttiContext;
        LType: TRttiType;
        LAttr: TCustomAttribute;
        Invoke : boolean;
        Meth: TRttiMethod;
        Param : TArray<TValue>;
      begin
        { Az Rtti kontextus ltrehozsa }
        LContext := TRttiContext.Create;
        { A tpusinformcik kiolvassa }
        LType := LContext.GetType(Instance.ClassType);
        { Az attribtum megkeresse s felhasznlsa }
        for LAttr in LType.GetAttributes do
          if LAttr is TInvarians then begin
            Meth := LType.GetMethod(TInvarians(LAttr).Vizsgal);
            Setlength(Param, 0);
	    { Az invarins ellenrzse, a visszaadott eredmny trolsa }
            Invoke := Meth.Invoke(Instance, Param).AsBoolean;
          end;
        for LAttr in Method.GetAttributes do
          if LAttr is TEloFelt then begin
            Meth := LType.GetMethod(TEloFelt(LAttr).Vizsgal);
            Setlength(Param, 0);
	    { Az elfelttel ellenrzse, a metdus hivsa a visszaadott eredmny s az invarins eredmnynek fggvnyben }
            DoInvoke := Invoke and Meth.Invoke(Instance, Param).AsBoolean;
          end;
        LContext.Free;
      end;

  { Az OnAfter esemny belltsa az itt deklarlt nvtelen eljrsra }
  FInterceptor.OnAfter :=
      procedure(Instance: TObject; Method: TRttiMethod;
          const Args: TArray<TValue>; var Result: TValue)
      var
        LContext: TRttiContext;
        LType: TRttiType;
        LAttr: TCustomAttribute;
        Meth: TRttiMethod;
        Param : TArray<TValue>;
      begin
        { A pldny felszabadtsa utn mr nem kellenek vizsglatok }
        if Method.Name = 'FreeInstance' then
          Exit;

        { Az Rtti kontextus ltrehozsa }
        LContext := TRttiContext.Create;
        { A tpusinformcik kiolvassa }
        LType := LContext.GetType(Instance.ClassType);
        { Az attribtum megkeresse s felhasznlsa }
        for LAttr in LType.GetAttributes do
          if LAttr is TInvarians then begin
            Meth := LType.GetMethod(TInvarians(LAttr).Vizsgal);
            Setlength(Param, 0);
	    { Az invarins ellenrzse }
            Meth.Invoke(Instance, Param);
          end;

        for LAttr in Method.GetAttributes do
          if LAttr is TUtoFelt then begin
            Meth := LType.GetMethod(TUtoFelt(LAttr).Vizsgal);
            Setlength(Param, 0);
	    { Az utfelttel ellenrzse }
            Meth.Invoke(Instance, Param);
          end;
        LContext.Free;
      end;

  { adott osztlypldny futsidej tpusnak megvltoztatsa }
  FInterceptor.Proxify (Instance);
end;

procedure THelyessegBizonyito<T>.Felold (Instance : T);
begin
	FInterceptor.UnProxify (Instance);
end;

end.
